#!/usr/bin/env python

import sys
from argparse import ArgumentParser
from socketlock import SocketLock
from settings import LOCK_NAME, JENKINS_URL, logger
from launchpad import LaunchpadTrigger
import os

LAUNCHPADLIB_DIR = os.path.expanduser('~/.launchpadlib-launchpadTrigger')


def trigger_ci_build(branch, job, jenkins_url):
    """Trigger a CI build."""

    launchpad_trigger = LaunchpadTrigger(LAUNCHPADLIB_DIR)
    mps = launchpad_trigger.get_merge_proposals(branch, ['Needs review'])
    launchpad_trigger.trigger_ci_build(mps, job, jenkins_url)


def trigger_al_build(branch, job, jenkins_url, fasttrack_merge):
    """Trigger an autolanding build."""

    launchpad_trigger = LaunchpadTrigger(LAUNCHPADLIB_DIR)
    mps = launchpad_trigger.get_merge_proposals(branch, ['Approved'])
    launchpad_trigger.trigger_al_build(mps, job, jenkins_url, fasttrack_merge)


def trigger_jenkins(branch,
                autoland=False,
                trigger_ci=False,
                job=None,
                jenkins_url=None,
                fasttrack_merge=False):
    """Trigger a Jenkins build."""

    if jenkins_url is None:
        jenkins_url = JENKINS_URL

    if trigger_ci:
        trigger_ci_build(branch, job, jenkins_url)
    elif autoland:
        trigger_al_build(branch, job, jenkins_url, fasttrack_merge)
    return 1


def main():

    parser = ArgumentParser(
        description="Trigger a Jenkins job resulting from a merge proposal.")
    parser.add_argument('-b', '--branch', required=True,
                        help="the proposed Launchpad branch to merge")
    parser.add_argument('-t', '--trigger-ci',
                        action='store_true',
                        help="trigger a 'CI' build")
    parser.add_argument('-l', '--autoland',
                        action='store_true',
                        help="trigger an autolanding build")
    parser.add_argument('-j', '--job', required=True,
                        help="name of the Jenkins job to trigger")
    parser.add_argument('-e', '--jenkins-url',
                        help="URL of the Jenkins job to trigger")
    parser.add_argument('-f', '--fasttrack-merge',
                        default=False,
                        action='store_true',
                        help='''skip the build process for simple branch
                        changes (i.e. changelog only updates)''')
    args = vars(parser.parse_args())

    if not args['trigger_ci'] and not args['autoland']:
        parser.error(
            'You need to specify one of --trigger-ci (-t) or --autoland (-l)')

    return trigger_jenkins(**args)


if __name__ == "__main__":
    # this is here because launchpadlib is not thread safe (yet)
    # (see also lp:459418 and lp:1025153) and also to limit the
    # amount of concurrent connections to launchpad
    logger.debug('Going to acquire launchpad lock for launchpadTrigger')
    trigger_lock_name = LOCK_NAME + '-launchpadTrigger'
    with SocketLock(trigger_lock_name):
        logger.debug('Lock acquired for launchpadTrigger')
        sys.exit(main())
