from mock import MagicMock, patch
import jenkinsutils
import unittest
from tests import JenkinsJSONData
from settings import PUBLIC_JENKINS_URL


class JenkinsUtilsGetDownstreamProjects(unittest.TestCase):
    def setUp(self):
        self.jenkins = MagicMock()
        self.jenkins.get_json_data = MagicMock(
                return_value={'downstreamProjects':
                                            [{'name': 'project1'},
                                             {'name': 'project2'},
                                             {'name': 'project3'}]})

    def test_get_downstream_projects(self):
        ret = jenkinsutils.get_downstream_projects(self.jenkins,
                                                   'http://ip/job/x/55')
        expected = ['project1', 'project2', 'project3']
        self.assertEqual(ret, expected)

    def test_get_downstream_projects_none(self):
        self.jenkins.get_json_data = MagicMock(
                return_value={'downstreamProjects': []})
        ret = jenkinsutils.get_downstream_projects(self.jenkins,
                                                   'http://ip/job/x')
        self.assertEqual(ret, [])

    def test_get_merge_job(self):
        self.jenkins.get_json_data = MagicMock(
                return_value={'downstreamProjects':
                              [{"name": "generic-cleanup"},
                               {"name": "generic-land"}]})
        ret = jenkinsutils.get_merge_job(self.jenkins, 'http://ip/job/x')
        self.assertEqual(ret, 'generic-land')

    def test_get_merge_job_none(self):
        self.jenkins.get_json_data = MagicMock(
                return_value={'downstreamProjects': []})
        with self.assertRaises(KeyError) as err:
            jenkinsutils.get_merge_job(self.jenkins, 'http://ip/job/x')
        self.assertIn("No merge jobs found", "%s" % (err.exception))

    def test_get_merge_job_too_many(self):
        self.jenkins.get_json_data = MagicMock(
                return_value={'downstreamProjects':
                              [{'name': 'my-land'},
                               {'name': 'your-land'}]})
        with self.assertRaises(KeyError) as err:
            jenkinsutils.get_merge_job(self.jenkins, 'http://ip/job/x')
        self.assertIn("Multiple merge jobs found: my-land, your-land",
                      "%s" % (err.exception))


def hide_jenkins_url(url):
    return url.replace('10.0.0.1', 'jenkins')


class JenkinsUtilsGetDownstreamBuilds(unittest.TestCase):

    def setUp(self):
        self.jenkins = JenkinsJSONData()

    def test_get_downstream_builds(self):
        ret = jenkinsutils.get_downstream_builds(self.jenkins,
                                                     'http://ip:8080/job/',
                                                     'my-job',
                                                     3)
        expected = [
                {'project': 'job-multiconfig',
                 'number': 3,
                 'url': 'http://ip:8080/job/job-multiconfig/3/',
                 'result': 'FAILURE', },
                {'project': 'job-freestyle',
                 'number': 7,
                 'url': 'http://ip:8080/job/job-freestyle/7/',
                 'result': 'SUCCESS', },
                ]
        self.assertEqual(ret, expected)


class JenkinsUtilsIsMatrixJob(unittest.TestCase):

    def setUp(self):
        self.jenkins = JenkinsJSONData()

    def test_matrix_job(self):
        ret = jenkinsutils.is_matrix_job(self.jenkins,
                'http://ip:8080/job/job-multiconfig/')
        self.assertEqual(ret, True)

    def test_matrix_job_with_build_number(self):
        ret = jenkinsutils.is_matrix_job(self.jenkins,
                'http://ip:8080/job/job-multiconfig/32')
        self.assertEqual(ret, True)

    def test_non_matrix_job(self):
        ret = jenkinsutils.is_matrix_job(self.jenkins,
                'http://ip:8080/job/job-freestyle/')
        self.assertEqual(ret, False)


class JenkinsUtilsGetExecutedBuilds(unittest.TestCase):
    def setUp(self):
        self.jenkins = JenkinsJSONData()
        self.build_url = self.jenkins.build_url

    def test_get_executed_builds_wrong_url(self):
        ret = jenkinsutils.get_executed_builds(self.jenkins,
                                                   'wrong url')
        expected = None
        self.assertEqual(ret, expected)

    def test_get_executed_builds(self):
        with patch('jenkinsutils.hide_jenkins_url', new=hide_jenkins_url):
            ret = jenkinsutils.get_executed_builds(self.jenkins,
                                                   self.build_url)
        expected = [
                {'output': 'http://ip:8080/job/job-multiconfig/.' +
                            '/distribution=raring,flavor=amd64/3/console',
                 'result': 'FAILURE', },
                {'output': 'http://ip:8080/job/job-multiconfig/.' +
                            '/distribution=raring,flavor=i386/3/console',
                 'result': 'FAILURE', },
                {'output': 'http://ip:8080/job/job-freestyle/7//console',
                 'result': u'SUCCESS', },
                ]
        self.assertEqual(ret, expected)


class JenkinsUtilsGetExecutedTestRuns(unittest.TestCase):
    def setUp(self):
        self.jenkins = JenkinsJSONData()

    def test_get_coverity_artifacts(self):
        with patch('jenkinsutils.hide_jenkins_url', new=hide_jenkins_url):
            ret = jenkinsutils.get_coverity_artifacts(self.jenkins,
                                    'http://ip:8080/job/job-multiconfig/' +
                                    './distribution=raring,flavor=amd64/3/')
        expected = ['http://ip:8080/job/job-multiconfig/./distribution=' +
                     'raring,flavor=amd64/3//artifact/results/coverity/' +
                     'CID_10895.html',
                   'http://ip:8080/job/job-multiconfig/./distribution=' +
                     'raring,flavor=amd64/3//artifact/results/coverity/' +
                     'CID_10896.html',
                   ]
        self.assertEqual(ret, expected)


class JenkinsUtilsIsJobPublished(unittest.TestCase):
    def test_is_job_published_with_private_job(self):
        jenkins = MagicMock()
        jenkins.get_job_config = MagicMock(return_value='<b></b>')
        with patch('jenkinsutils.jenkins.Jenkins',
                   new=MagicMock(return_value=jenkins)):
            self.assertFalse(jenkinsutils.is_job_published('my-job'))

    def test_is_job_published_with_no_job(self):
        jenkins = MagicMock()
        jenkins.get_job_config = MagicMock(return_value='')
        with patch('jenkinsutils.jenkins.Jenkins',
                   new=MagicMock(return_value=jenkins)):
            self.assertFalse(jenkinsutils.is_job_published(None))

    def test_is_job_published_with_public_job(self):
        jenkins = MagicMock()
        jenkins.get_job_config = MagicMock(
           return_value='<hudson.plugins.build__publisher.BuildPublisher>' +
                        '</hudson.plugins.build__publisher.BuildPublisher>')
        with patch('jenkinsutils.jenkins.Jenkins',
                   new=MagicMock(return_value=jenkins)):
            self.assertTrue(jenkinsutils.is_job_published('my-job'))

    def test_is_job_published_with_exception(self):
        jenkins = MagicMock()
        jenkins.get_job_config.side_effect = Exception(
                                                 'get_job_config exception')
        with patch('jenkinsutils.jenkins.Jenkins',
                   new=MagicMock(return_value=jenkins)):
            self.assertFalse(jenkinsutils.is_job_published('my-job'))


class JenkinsUtilsTestHideJenkinsUrl(unittest.TestCase):

    def test_hide_jenkins_url_public_url(self):
        urls = [
                ('http://10.0.0.1:8080/job/android-audiosystem/31/',
                 PUBLIC_JENKINS_URL + '/job/android-audiosystem/31/'),
                ('http://10.0.0.1:8080/job/android-ubuntu-matrix/4/',
                 PUBLIC_JENKINS_URL + '/job/android-ubuntu-matrix/4/'),
                ('https://10.0.0.1:8080/job/unity-ci/32/',
                 PUBLIC_JENKINS_URL + '/job/unity-ci/32/')
               ]

        is_job_published = lambda job: True
        for url, expected in urls:
            with patch('jenkinsutils.is_job_published',
                       new=is_job_published):
                self.assertEqual(jenkinsutils.hide_jenkins_url(url),
                                 expected)

    def test_hide_jenkins_url_hidden_url(self):
        urls = [
                'https://10.0.0.1:8080/job/android-audiosystem/31/',
                'http://10.0.0.1:8080/job/android-ubuntu-matrix/4/',
                'http://10.0.0.1:8080/job/unity-ci/32/',
               ]

        is_job_published = lambda job: False
        for url in urls:
            with patch('jenkinsutils.is_job_published',
                       new=is_job_published):
                self.assertEqual(jenkinsutils.hide_jenkins_url(url),
                           url.replace('http://10.0.0.1:8080',
                                       'http://jenkins:8080'))

    def test_hide_jenkins_url_unknown_url(self):
        urls = [
                'http://12.0.0.2:8080/job/android-audiosystem/31/',
                'http://12.0.0.2:8080/job/android-ubuntu-matrix/4/',
                'https://12.0.0.2:8080/job/unity-ci/32/',
                'https://12.0.0.080/droid-audiosystem/31/',
                'http://12.0.0.080/droid-ubuntu-matrix/4/',
                'http://12.0.0.080/ity-ci/32/',
               ]

        is_job_published = lambda job: False
        for url in urls:
            with patch('jenkinsutils.is_job_published',
                       new=is_job_published):
                self.assertEqual(jenkinsutils.hide_jenkins_url(url),
                                 url)


class JenkinsUtilsGetParameters(unittest.TestCase):

    jenkins_params = {"property":
                      [{},
                       {"parameterDefinitions":
                        [{"defaultParameterValue":
                          {"name": "target_branch", "value": "lp:unity"},
                          "description": "lp target branch",
                          "name": "target_branch",
                          "type": "StringParameterDefinition"},
                         {"defaultParameterValue":
                          {"name": "packaging_branch", "value": ""},
                          "description": "lp packaging branch",
                          "name": "packaging_branch",
                          "type": "StringParameterDefinition"},
                         {"defaultParameterValue":
                          {"name": "ppa_target",
                           "value": "ppa:unity-team/staging"},
                          "description": "lp PPA",
                          "name": "ppa_target",
                          "type": "StringParameterDefinition"},
                         {"defaultParameterValue":
                          {"name": "distributions",
                           "value": "quantal,raring"},
                          "description": "Distributions to dput",
                          "name": "distributions",
                          "type": "StringParameterDefinition"},
                         {"defaultParameterValue":
                          {"name": "use_description_for_commit",
                           "value": True},
                          "description": "use description",
                          "name": "use_description_for_commit",
                          "type": "BooleanParameterDefinition"},
                         {"defaultParameterValue":
                          {"name": "version_string_format",
                           "value": "{UPSTREAM_VERSION}bzr{BZR_REVISION}"},
                          "description": "This is a template",
                          "name": "version_string_format",
                          "type": "StringParameterDefinition"},
                         {"defaultParameterValue":
                          {"name": "dput_hooks",
                           "value": "H10disable_powerpc"},
                          "description": "hack hooks used for dput",
                          "name": "dput_hooks",
                          "type": "StringParameterDefinition"}]}]}

    def setUp(self):
        self.jenkins = JenkinsJSONData()
        self.jenkins.get_json_data = MagicMock(
                return_value=self.jenkins_params)

    def test_get_parameter(self):
        ret = jenkinsutils.get_parameter(self.jenkins, 'http://ip/job/x',
                                         'target_branch')
        self.assertEqual(ret, 'lp:unity')

    def test_get_parameter_boolean(self):
        ret = jenkinsutils.get_parameter(self.jenkins, 'http://ip/job/x',
                                         'use_description_for_commit')
        self.assertTrue(ret)

    def test_get_parameter_default(self):
        ret = jenkinsutils.get_parameter(self.jenkins, 'http://ip/job/x',
                                         'missing_branch', 'lp:missing')
        self.assertEqual(ret, 'lp:missing')

    def test_get_parameter_key_error_exception(self):
        with self.assertRaises(KeyError) as err:
            ret = jenkinsutils.get_parameter(self.jenkins, 'http://ip/job/x',
                                             'missing_branch')
        self.assertIn("jenkins job parameter [missing_branch]: not found",
                      "%s" % (err.exception))
